/* 
 *  vcdpad pads VCD MPEGs to be usable by mkvcdfs
 *
 *  Copyright (C) 2000 Stefan Berndtsson <stefan@nocrew.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *  Thanks to Brian King for finding a solution to the audio blips.
 *
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <fcntl.h>

#define FSIZE 2324

unsigned char buffer[FSIZE*3];
int start,end;
int fd;

int fill_buffer(int offset)
{
  int size;

  lseek(fd, offset, SEEK_SET);
  size = read(fd, buffer, FSIZE*2);
  start = offset;
  end = offset + size;
  
  if(size != FSIZE*2)
    return -1;
  return 0;
}

int get_byte(int offset)
{
  int endpoint;

  if((offset >= start) &&
     (offset < end))
    return (int) buffer[offset - start];

  endpoint = fill_buffer(offset);

  if(endpoint == -1)
    return -1;
  
  return (int) buffer[0];
}

int find_frame(int offset, int totsize)
{
  int i;

  for(i=offset;i<totsize;i++) {
    if(get_byte(i) == -1)
      return -1;
    if(get_byte(i+3) == -1)
      return -1;

    if((get_byte(i+0) == 0) &&
       (get_byte(i+1) == 0) &&
       (get_byte(i+2) == 1) &&
       (get_byte(i+3) == 0xba) &&
       (get_byte(i+4) == 0x21))  // Added to get rid of audio blips
      return i;
  }
  
  fprintf(stderr, "I don't think this point should've been reached\n");
  return -2;
}

int main(int argc, char *argv[])
{
  int ofd,endpoint;
  int block,sum=0;
  int i;
  char *input = "input.mpg";
  char *output = "output.mpg";
  int inputsize, lastframe, offset;
  static unsigned char buf[FSIZE*3];
  static unsigned char out[FSIZE*3];
  int pad_count = 0;

  if(argc == 2) {
    input = argv[1];
  } else if(argc == 3) {
    input = argv[1];
    output = argv[2];
  } else {
    fprintf(stdout, "Usage: %s [input [output]]\n", argv[0]);
    exit(-1);
  }

  if((fd = open(input, O_RDONLY)) < 0) {
    perror(input);
    exit(-1);
  }

  inputsize = lseek(fd, 0, SEEK_END);
  lseek(fd, 0, SEEK_SET);
  
  if((ofd = open(output, O_WRONLY|O_CREAT|O_TRUNC, 00644)) < 0) {
    perror(output);
    exit(-1);
  }

  for(i=0;i<FSIZE;i++) {
    out[i] = 0;
  }

  fill_buffer(0);

  endpoint = 0;
  lastframe = -1;

  while(!endpoint) {
    fprintf(stderr, "\rWritten %dK", sum/1024);
    fflush(stderr);

    if(lastframe == -1) {
      lastframe = find_frame(0, inputsize);
    } else {
      offset = find_frame(lastframe+1, inputsize);

      if(offset == -1) {
	endpoint = 1;
	break;
      }
      
      for(i=0;i<FSIZE;i++) {
	buf[i] = get_byte(i+lastframe);
      }
      
      block = offset - lastframe;
      // fprintf(stdout, "Block size: %d\n", block);
      lastframe = offset;
      
      if(block == FSIZE) {
	sum += FSIZE;
	write(ofd, buf, FSIZE);
	lseek(fd, -FSIZE, SEEK_CUR);
      }
      
      if((block > 0) && (block < FSIZE)) {
	if (block != 2304)
	{
	    fprintf(stdout, "Block size: %d\n", block);
	    fprintf(stdout, "Writing block of %d bytes\n", block);
	}
	pad_count++;
	sum += FSIZE;
	write(ofd, buf, block);
	write(ofd, out, FSIZE-block);
	lseek(fd, -(FSIZE+(FSIZE-block)), SEEK_CUR);
      }
    }
  }
  fprintf(stderr, "\nDone padding.\n");
  fprintf(stdout, "\nPadded %d blocks\n", pad_count);
  close(fd);
  close(ofd);

  return 0;
}
